@@ -62,7 +62,7 @@ kevent_fflags_dump(const struct kevent *kev)
         KEVFFL_DUMP(NOTE_FFCOPY);
         KEVFFL_DUMP(NOTE_TRIGGER);
     }  else {
-        strncat((char *) &buf[0], " ", 1);
+        buf[0] = ' ';
     }
     buf[strlen(buf) - 1] = ')';
 
@@ -101,7 +101,7 @@ kevent_flags_dump(const struct kevent *kev)
 const char *
 kevent_dump(const struct kevent *kev)
 {
-    static __thread char buf[1024];
+    static __thread char buf[2147];
 
     snprintf((char *) &buf[0], sizeof(buf), 
             "{ ident=%d, filter=%s, %s, %s, data=%d, udata=%p }",
@@ -144,7 +144,7 @@ kevent_copyin_one(struct kqueue *kq, const struct kevent *src)
             memcpy(&kn->kev, src, sizeof(kn->kev));
             kn->kev.flags &= ~EV_ENABLE;
             kn->kev.flags |= EV_ADD;//FIXME why?
-			kn->kn_kq = kq;
+            kn->kn_kq = kq;
             assert(filt->kn_create);
             if (filt->kn_create(filt, kn) < 0) {
                 knote_release(kn);
@@ -198,23 +198,17 @@ kevent_copyin(struct kqueue *kq, const struct kevent *src, int nchanges,
 
     dbg_printf("nchanges=%d nevents=%d", nchanges, nevents);
 
-    /* TODO: refactor, this has become convoluted to support EV_RECEIPT */
     for (nret = 0; nchanges > 0; src++, nchanges--) {
 
         if (kevent_copyin_one(kq, src) < 0) {
             dbg_printf("errno=%s",strerror(errno));
             status = errno;
-            goto err_path;
+        } else if (src->flags & EV_RECEIPT) {
+            status = 0;
         } else {
-            if (src->flags & EV_RECEIPT) {
-                status = 0;
-                goto err_path;
-            }
+            continue;
         }
 
-        continue;
-
-err_path:
         if (nevents > 0) {
             memcpy(eventlist, src, sizeof(*src));
             eventlist->data = status;
