@@ -63,16 +63,27 @@ struct eventfd {
 #endif
 };
 
-/* 
+/*
  * Flags used by knote->kn_flags
  */
-#define KNFL_PASSIVE_SOCKET  (0x01)  /* Socket is in listen(2) mode */
-#define KNFL_REGULAR_FILE    (0x02)  /* File descriptor is a regular file */
-#define KNFL_KNOTE_DELETED   (0x10)  /* The knote object is no longer valid */
- 
+#define KNFL_FILE                (1U << 0U)
+#define KNFL_PIPE                (1U << 1U)
+#define KNFL_BLOCKDEV            (1U << 2U)
+#define KNFL_CHARDEV             (1U << 3U)
+#define KNFL_SOCKET_PASSIVE      (1U << 4U)
+#define KNFL_SOCKET_STREAM       (1U << 5U)
+#define KNFL_SOCKET_DGRAM        (1U << 6U)
+#define KNFL_SOCKET_RDM          (1U << 7U)
+#define KNFL_SOCKET_SEQPACKET    (1U << 8U)
+#define KNFL_KNOTE_DELETED       (1U << 31U)
+#define KNFL_SOCKET              (KNFL_SOCKET_STREAM |\
+                                  KNFL_SOCKET_DGRAM |\
+                                  KNFL_SOCKET_RDM |\
+                                  KNFL_SOCKET_SEQPACKET)
+
 struct knote {
     struct kevent     kev;
-    int               kn_flags;       
+    unsigned int      kn_flags;
     union {
         /* OLD */
         int           pfd;       /* Used by timerfd */
@@ -81,7 +92,7 @@ struct knote {
             nlink_t   nlink;  /* Used by vnode */
             off_t     size;   /* Used by vnode */
         } vnode;
-        timer_t       timerid;  
+        timer_t       timerid;
         struct sleepreq *sleepreq; /* Used by posix/timer.c */
 		void          *handle;      /* Used by win32 filters */
     } data;
@@ -113,7 +124,7 @@ struct filter {
     /* knote operations */
 
     int     (*kn_create)(struct filter *, struct knote *);
-    int     (*kn_modify)(struct filter *, struct knote *, 
+    int     (*kn_modify)(struct filter *, struct knote *,
                             const struct kevent *);
     int     (*kn_delete)(struct filter *, struct knote *);
     int     (*kn_enable)(struct filter *, struct knote *);
@@ -141,7 +152,7 @@ struct filter {
 struct kqueue {
     int             kq_id;
     struct filter   kq_filt[EVFILT_SYSCOUNT];
-    fd_set          kq_fds, kq_rfds; 
+    fd_set          kq_fds, kq_rfds;
     int             kq_nfds;
     tracing_mutex_t kq_mtx;
     volatile uint32_t kq_ref;
@@ -183,6 +194,7 @@ extern const struct kqueue_vtable kqops;
 /*
  * knote internal API
  */
+int knote_free_all(struct filter *filt);
 struct knote * knote_lookup(struct filter *, uintptr_t);
 //DEADWOOD: struct knote * knote_get_by_data(struct filter *filt, intptr_t);
 struct knote * knote_new(void);
