@@ -22,13 +22,18 @@
 const struct filter evfilt_proc = EVFILT_NOTIMPL;
 
 int
-kevent_wait(struct kqueue *kq, const struct timespec *timeout)
+posix_kevent_wait(
+        struct kqueue *kq, 
+        const struct timespec *timeout)
 {
-    int n;
+    int n, nfds;
+    fd_set rfds;
+
+    nfds = kq->kq_nfds;
+    rfds = kq->kq_fds;
 
     dbg_puts("waiting for events");
-    kq->kq_rfds = kq->kq_fds;
-    n = pselect(kq->kq_nfds, &kq->kq_rfds, NULL , NULL, timeout, NULL);
+    n = pselect(nfds, &rfds, NULL , NULL, timeout, NULL);
     if (n < 0) {
         if (errno == EINTR) {
             dbg_puts("signal caught");
@@ -38,11 +43,13 @@ kevent_wait(struct kqueue *kq, const struct timespec *timeout)
         return (-1);
     }
 
+    kq->kq_rfds = rfds;
+
     return (n);
 }
 
 int
-kevent_copyout(struct kqueue *kq, int nready,
+posix_kevent_copyout(struct kqueue *kq, int nready,
         struct kevent *eventlist, int nevents)
 {
     struct filter *filt;
