@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 Mark Heily <mark@heily.com>
+ * Copyright (c) 2011 Mark Heily <mark@heily.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -14,42 +14,36 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#include <errno.h>
-#include <fcntl.h>
-#include <pthread.h>
-#include <signal.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <sys/queue.h>
-#include <sys/socket.h>
-#include <sys/types.h>
-#include <string.h>
-#include <unistd.h>
-
-#include "sys/event.h"
 #include "private.h"
 
 /* Highest signal number supported. POSIX standard signals are < 32 */
 #define SIGNAL_MAX      32
 
-struct sentry {
-    size_t          s_cnt;
-    struct filter  *s_filt;
-    struct knote   *s_knote;
-};
+static struct sentry {
+    int             st_signum;
+    int             st_port;
+    volatile sig_atomic_t st_count;
+    struct kevent st_kev;
+} sigtbl[SIGNAL_MAX];
 
 static pthread_mutex_t sigtbl_mtx = PTHREAD_MUTEX_INITIALIZER;
-static struct sentry sigtbl[SIGNAL_MAX];
 
 static void
 signal_handler(int sig)
 {
-    struct sentry *s = &sigtbl[sig];
-    ssize_t n;
-
-    dbg_printf("caught sig=%d", sig);
-    atomic_inc(&s->s_cnt);
-    n = write(s->s_filt->kf_wfd, &sig, sizeof(sig));//FIXME:errhandling
+    struct sentry *s;
+   
+    if (sig < 0 || sig >= SIGNAL_MAX) // 0..31 are valid
+    {    
+        dbg_printf("Received unexpected signal %d", sig);
+        return;
+    }
+    
+    s = &sigtbl[sig];
+    dbg_printf("sig=%d %d", sig, s->st_signum);
+    atomic_inc((volatile uint32_t *) &s->st_count);
+    port_send(s->st_port, X_PORT_SOURCE_SIGNAL, &sigtbl[sig]);
+    /* TODO: crash if port_send() fails? */
 }
 
 static int
@@ -68,13 +62,16 @@ catch_signal(struct filter *filt, struct knote *kn)
 		dbg_perror("sigaction");
 		return (-1);
 	}
-    /* FIXME: will clobber previous entry, if any */
+
     pthread_mutex_lock(&sigtbl_mtx);
-    sigtbl[kn->kev.ident].s_filt = filt;
-    sigtbl[kn->kev.ident].s_knote = kn;
+    sigtbl[sig].st_signum = sig;
+    sigtbl[sig].st_port = filter_epfd(filt);
+    sigtbl[sig].st_count = 0;
+    memcpy(&sigtbl[sig].st_kev, &kn->kev, sizeof(struct kevent));
     pthread_mutex_unlock(&sigtbl_mtx);
 
     dbg_printf("installed handler for signal %d", sig);
+    dbg_printf("sigtbl ptr = %p", &sigtbl[sig]);
     return (0);
 }
 
@@ -91,32 +88,17 @@ ignore_signal(int sig)
 		dbg_perror("sigaction");
 		return (-1);
 	}
-    pthread_mutex_lock(&sigtbl_mtx);
-    sigtbl[sig].s_filt = NULL;
-    sigtbl[sig].s_knote = NULL;
-    pthread_mutex_unlock(&sigtbl_mtx);
 
     dbg_printf("removed handler for signal %d", sig);
     return (0);
 }
 
-int
-evfilt_signal_init(struct filter *filt)
-{
-    return filter_socketpair(filt);
-}
-
-void
-evfilt_signal_destroy(struct filter *filt)
-{
-    close(filt->kf_pfd);
-}
-
 int
 evfilt_signal_knote_create(struct filter *filt, struct knote *kn)
 {
     if (kn->kev.ident >= SIGNAL_MAX) {
-        dbg_printf("unsupported signal number %u", (u_int) kn->kev.ident);
+        dbg_printf("unsupported signal number %u", 
+                    (unsigned int) kn->kev.ident);
         return (-1);
     }
 
@@ -126,7 +108,7 @@ evfilt_signal_knote_create(struct filter *filt, struct knote *kn)
 }
 
 int
-evfilt_signal_knote_modify(struct filter *filt, struct knote *kn, 
+evfilt_signal_knote_modify(struct filter *filt UNUSED, struct knote *kn, 
                 const struct kevent *kev)
 {
     kn->kev.flags = kev->flags | EV_CLEAR;
@@ -134,7 +116,7 @@ evfilt_signal_knote_modify(struct filter *filt, struct knote *kn,
 }
 
 int
-evfilt_signal_knote_delete(struct filter *filt, struct knote *kn)
+evfilt_signal_knote_delete(struct filter *filt UNUSED, struct knote *kn)
 {   
     return ignore_signal(kn->kev.ident);
 }
@@ -146,49 +128,37 @@ evfilt_signal_knote_enable(struct filter *filt, struct knote *kn)
 }
 
 int
-evfilt_signal_knote_disable(struct filter *filt, struct knote *kn)
+evfilt_signal_knote_disable(struct filter *filt UNUSED, struct knote *kn)
 {
     return ignore_signal(kn->kev.ident);
 }
 
 int
-evfilt_signal_copyout(struct filter *filt, 
-            struct kevent *dst, 
-            int nevents)
+evfilt_signal_copyout(struct kevent *dst, struct knote *src, void *ptr)
 {
-    struct sentry *s;
-    struct knote *kn;
-    int sig;
-    ssize_t n;
+    port_event_t *pe = (port_event_t *) ptr;
+    struct sentry *ent = (struct sentry *) pe->portev_user;
 
-    n = read(filt->kf_pfd, &sig, sizeof(sig));//FIXME:errhandling
-    dbg_printf("got sig=%d", sig);
-    s = &sigtbl[sig];
-    kn = s->s_knote;
-    //TODO: READ counter: s->s_knote->kev.data = ?;
-    /* TODO: dst->data should be the number of times the signal occurred */
-    dst->ident = sig;
+    pthread_mutex_lock(&sigtbl_mtx);
+    dbg_printf("sigtbl ptr = %p sig=%d", ptr, ent->st_signum);
+    dst->ident = ent->st_kev.ident;
     dst->filter = EVFILT_SIGNAL;
-    dst->udata = kn->kev.udata;
-    dst->flags = kn->kev.flags; 
+    dst->udata = ent->st_kev.udata;
+    dst->flags = ent->st_kev.flags; 
     dst->fflags = 0;
-    dst->data = 1;  
+    dst->data = 1;
+    pthread_mutex_unlock(&sigtbl_mtx);
 
-    if (kn->kev.flags & EV_DISPATCH) {
-        ignore_signal(kn->kev.ident);
-        KNOTE_DISABLE(kn);
-    } else if (kn->kev.flags & EV_ONESHOT) {
-        ignore_signal(kn->kev.ident);
-        knote_free(filt, kn);
-    }
+    if (src->kev.flags & EV_DISPATCH || src->kev.flags & EV_ONESHOT) 
+        ignore_signal(src->kev.ident);
 
     return (1);
 }
 
 const struct filter evfilt_signal = {
     EVFILT_SIGNAL,
-    evfilt_signal_init,
-    evfilt_signal_destroy,
+    NULL,
+    NULL,
     evfilt_signal_copyout,
     evfilt_signal_knote_create,
     evfilt_signal_knote_modify,
